@@ -1,12 +1,12 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "gh-notify" "2.1.8"
+(define-package "gh-notify" "20251016.20"
   "A veneer for Magit/Forge GitHub notifications."
   '((emacs "29.1")
     (magit "3.3.0")
     (forge "0.4.0"))
   :url "https://github.com/anticomputer/gh-notify"
-  :commit "d606d1390778cb104c28dbc5220e685293e1e687"
-  :revdesc "d606d1390778"
+  :commit "6a25fd83f9e29939e0b49ecfaeb20e94386e42e6"
+  :revdesc "6a25fd83f9e2"
   :keywords '("comm")
   :authors '(("Bas Alberts" . "bas@anti.computer")
              ("xristos" . "xristos@sdf.org"))
