@@ -8,8 +8,8 @@
 ;; All rights reserved
 
 ;; Modified: 2024-04-10
-;; Package-Version: 2.1.8
-;; Package-Revision: d606d1390778
+;; Package-Version: 20251016.20
+;; Package-Revision: 6a25fd83f9e2
 ;; Author: Bas Alberts <bas@anti.computer>
 ;;         xristos <xristos@sdf.org>
 ;;
@@ -197,11 +197,8 @@ to use `gh-notify-display-state' via a keybinding instead.")
   "Show notification titles when equal to :title, URLs otherwise.
 This can be toggled by `gh-notify-toggle-url-view'.")
 
-(defvar gh-notify-reason-limit '(:all)
-  "Default display limit.
-Can be a list of reason keywords that will be combined with OR logic.
-Supported reasons: :assign, :mention, :team_mention, :subscribed,
-:author, :comment, :review-requested, :mark, :unread, :all.")
+(defvar gh-notify-reason-limit :all
+  "Default display limit.")
 
 (defvar gh-notify-default-repo-limit '()
   "List of default repo limits.
@@ -222,6 +219,9 @@ Repos are in the form \"owner/repo\".
 
 Use this to muzzle specific repos that you want to silence across sessions.")
 
+(defvar gh-notify-smokescreen-path (expand-file-name "~/.gh-notify-smokescreen")
+  "The default path for the magit `forge-visit' smokescreen repo.")
+
 (defvar gh-notify-redraw-on-visit t
   "Automatically redraw notifications on `forge-visit'.
 
@@ -291,20 +291,19 @@ NOTIFICATIONS must be an alist as returned from `gh-notify-get-notifications'."
     for ts = (encode-time (iso8601-parse (oref forge-notification updated)))
     for date = (format-time-string "%F" ts) ; use local time on our end for display
     for type = (oref forge-notification type)
-    for topic = (oref forge-notification topic)
-    for topic-obj = (and topic
-                         (pcase type
-                           ('discussion
-                            (forge-get-discussion topic))
-                           ('issue
-                            (forge-get-issue topic))
-                           ('pullreq
-                            (forge-get-pullreq topic))))
+    for topic-obj = (pcase type
+                      ('discussion
+                       (forge-get-discussion (oref forge-notification topic)))
+                      ('issue
+                       (forge-get-issue (oref forge-notification topic)))
+                      ('pullreq
+                       (forge-get-pullreq (oref forge-notification topic))))
     for status = (when topic-obj (oref topic-obj status))
     for number = (when topic-obj (oref topic-obj number))
     for id = (oref forge-notification id)
     for reason = (oref forge-notification reason)
     for updated = (oref forge-notification updated)
+    for topic = (oref forge-notification topic)
     for url = (oref forge-notification url)
     for title = (oref forge-notification title)
     for state = (when gh-notify-show-state
@@ -326,7 +325,8 @@ NOTIFICATIONS must be an alist as returned from `gh-notify-get-notifications'."
              :type type
              :repo-id repo-id
              :repo repo
-             ;; Modern Forge uses topic status scheme (unread/pending/done)
+             ;; unread-p changed to a topic status scheme in db version >= 11
+             ;; https://github.com/magit/forge/blob/99d319823719339e0c324ad3e9f78564865ec07a/lisp/forge-db.el#L471
              :unread (eq status 'unread)
              :status status
              :url url
@@ -505,10 +505,7 @@ NOTIFICATIONS must be an alist as returned from `gh-notify-get-notifications'."
        (when gh-notify--unread-limit
          (format "%s " gh-notify--unread-limit))
        (when gh-notify--type-limit (format "type: %s " gh-notify--type-limit))
-       (format "reason: %s "
-               (mapconcat (lambda (r) (substring (symbol-name r) 1))
-                          gh-notify-reason-limit
-                          ","))
+       (format "reason: %s " gh-notify-reason-limit)
        ;; if it's active, you already know which repos are in the filter, if not you don't care
        (when gh-notify--repo-limit ":repo ")
        (when gh-notify-show-timing
@@ -692,25 +689,25 @@ Type \\[gh-notify-limit-issue] to limit to issue notifications.
 
 Type \\[gh-notify-limit-pr] to limit to pull request notifications.
 
-Type \\[gh-notify-limit-assign] to toggle assign notifications.
+Type \\[gh-notify-limit-assign] to limit to assign notifications.
 
-Type \\[gh-notify-limit-author] to toggle author notifications.
+Type \\[gh-notify-limit-author] to limit to author notifications.
 
-Type \\[gh-notify-limit-mention] to toggle mention notifications.
+Type \\[gh-notify-limit-mention] to limit to mention notifications.
 
-Type \\[gh-notify-limit-team-mention] to toggle team-mention notifications.
+Type \\[gh-notify-limit-team-mention] to limit to team-mention notifications.
 
-Type \\[gh-notify-limit-subscribed] to toggle subscribed notifications.
+Type \\[gh-notify-limit-subscribed] to limit to subscribed notifications.
 
-Type \\[gh-notify-limit-comment] to toggle comment notifications.
+Type \\[gh-notify-limit-comment] to limit to comment notifications.
 
-Type \\[gh-notify-limit-review-requested] to toggle review-requested notifications.
+Type \\[gh-notify-limit-review-requested] to limit to review-requested notifications.
 
-Type \\[gh-notify-limit-marked] to toggle marked notifications.
+Type \\[gh-notify-limit-none] to remove any active reason limit.
 
-All reason filters can be stacked - multiple reasons can be active at once.
+Type \\[gh-notify-limit-marked] to only show marked notifications.
 
-Type \\[gh-notify-limit-none] to remove all reason filters and show all notifications.
+Type \\[gh-notify-limit-none] to remove the current limit and show all notifications.
 
 Marking:
 
@@ -850,9 +847,6 @@ It must not span more than one line but it may contain text properties."
   "Limits NOTIFICATION by status.
 Limiting operation depends on `gh-notify-reason-limit', `gh-notify-type-limit'
 and `gh-notify--repo-limit'."
-  ;; Ensure backward compatibility: convert old single-value format to list
-  (unless (listp gh-notify-reason-limit)
-    (setq-local gh-notify-reason-limit (list gh-notify-reason-limit)))
   (let ((repo-id (gh-notify-notification-repo-id notification)))
     ;; 3 pass filter: repo -> type -> reason
     (when
@@ -870,26 +864,17 @@ and `gh-notify--repo-limit'."
        (or (eq (gh-notify-notification-type notification) gh-notify--type-limit)
            (not gh-notify--type-limit))
        ;; reason filters (layer 3 filter)
-       ;; Check if notification matches any of the active reason filters
-       (or (member :all gh-notify-reason-limit)
-           (and (member :mark gh-notify-reason-limit)
-                (gh-notify-notification-is-marked notification))
-           (and (member :unread gh-notify-reason-limit)
-                (gh-notify-notification-unread notification))
-           (and (member :assign gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'assign))
-           (and (member :mention gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'mention))
-           (and (member :team_mention gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'team_mention))
-           (and (member :subscribed gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'subscribed))
-           (and (member :author gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'author))
-           (and (member :review-requested gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'review_requested))
-           (and (member :comment gh-notify-reason-limit)
-                (eq (gh-notify-notification-reason notification) 'comment)))))))
+       (cl-case gh-notify-reason-limit
+         (:all t)
+         (:mark (gh-notify-notification-is-marked notification))
+         (:unread (gh-notify-notification-unread notification))
+         (:assign (eq (gh-notify-notification-reason notification) 'assign))
+         (:mention (eq (gh-notify-notification-reason notification) 'mention))
+         (:team_mention (eq (gh-notify-notification-reason notification) 'team_mention))
+         (:subscribed (eq (gh-notify-notification-reason notification) 'subscribed))
+         (:author (eq (gh-notify-notification-reason notification) 'author))
+         (:review-requested (eq (gh-notify-notification-reason notification) 'review_requested))
+         (:comment (eq (gh-notify-notification-reason notification) 'comment)))))))
 
 (defun gh-notify-filter-notification (notification)
   "Filters NOTIFICATION using a case-insensitive match on either URL or title."
@@ -1010,7 +995,8 @@ All pull requests on prefix P."
   (cl-assert (eq major-mode 'gh-notify-mode) t)
   (when-let ((notification (gh-notify-current-notification)))
     ;; build a closure for a pull topic callback
-    (let* ((repo-id (gh-notify-notification-repo-id notification))
+    (let* ((default-directory gh-notify-smokescreen-path)
+           (repo-id (gh-notify-notification-repo-id notification))
            (repo (gh-notify-notification-repo notification))
            (P P)
            (callback
@@ -1033,8 +1019,8 @@ All pull requests on prefix P."
                     (let ((topic (and (string-match "^#\\([0-9]+\\) " choice)
                                       (string-to-number (match-string 1 choice)))))
                       (with-demoted-errors "Warning: %S"
-                        (when-let ((topic-obj (forge-get-pullreq repo topic)))
-                          (forge-topic-setup-buffer topic-obj))))))))))
+                        (with-temp-buffer
+                          (forge-visit-pullreq (forge-get-pullreq repo topic)))))))))))
       (forge--pull repo callback))))
 
 (defun gh-notify-ls-issues-at-point (P)
@@ -1044,7 +1030,8 @@ All issues on prefix P."
   (cl-assert (eq major-mode 'gh-notify-mode) t)
   (when-let ((notification (gh-notify-current-notification)))
     ;; build a closure for a pull topic callback
-    (let* ((repo-id (gh-notify-notification-repo-id notification))
+    (let* ((default-directory gh-notify-smokescreen-path)
+           (repo-id (gh-notify-notification-repo-id notification))
            (repo (gh-notify-notification-repo notification))
            (P P)
            (callback
@@ -1067,8 +1054,8 @@ All issues on prefix P."
                     (let ((topic (and (string-match "^#\\([0-9]+\\) " choice)
                                       (string-to-number (match-string 1 choice)))))
                       (with-demoted-errors "Warning: %S"
-                        (when-let ((topic-obj (forge-get-issue repo topic)))
-                          (forge-topic-setup-buffer topic-obj))))))))))
+                        (with-temp-buffer
+                          (forge-visit-issue (forge-get-issue repo topic)))))))))))
       (forge--pull repo callback))))
 
 (defun gh-notify-display-state ()
@@ -1107,21 +1094,10 @@ All issues on prefix P."
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-marked ()
-  "Toggle showing marked notifications.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show marked notifications."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :mark gh-notify-reason-limit)
-      ;; Remove :mark from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :mark gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :mark and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :mark (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :mark)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-type-none ()
@@ -1158,129 +1134,52 @@ Multiple reason filters can be active at once."
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-assign ()
-  "Toggle showing notifications with reason: assign.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: assign."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :assign gh-notify-reason-limit)
-      ;; Remove :assign from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :assign gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :assign and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :assign (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :assign)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-author ()
-  "Toggle showing notifications with reason: author.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: author."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :author gh-notify-reason-limit)
-      ;; Remove :author from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :author gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :author and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :author (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :author)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-mention ()
-  "Toggle showing notifications with reason: mention.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: mention."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :mention gh-notify-reason-limit)
-      ;; Remove :mention from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :mention gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :mention and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :mention (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :mention)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-team-mention ()
-  "Toggle showing notifications with reason: team_mention.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: team_mentioned."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :team_mention gh-notify-reason-limit)
-      ;; Remove :team_mention from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :team_mention gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :team_mention and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :team_mention (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :team_mention)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-subscribed ()
-  "Toggle showing notifications with reason: subscribed.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: subscribed."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :subscribed gh-notify-reason-limit)
-      ;; Remove :subscribed from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :subscribed gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :subscribed and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :subscribed (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :subscribed)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-comment ()
-  "Toggle showing notifications with reason: comment.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: comment."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :comment gh-notify-reason-limit)
-      ;; Remove :comment from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :comment gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :comment and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :comment (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :comment)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-review-requested ()
-  "Toggle showing notifications with reason: review_requested.
-When toggled on, removes :all from the filter list if present.
-Multiple reason filters can be active at once."
+  "Only show notifications with reason: review_requested."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (if (member :review-requested gh-notify-reason-limit)
-      ;; Remove :review-requested from the list
-      (progn
-        (setq-local gh-notify-reason-limit (remove :review-requested gh-notify-reason-limit))
-        ;; If list is empty, add :all back
-        (when (null gh-notify-reason-limit)
-          (setq-local gh-notify-reason-limit '(:all))))
-    ;; Add :review-requested and remove :all
-    (setq-local gh-notify-reason-limit
-                (cons :review-requested (remove :all gh-notify-reason-limit))))
+  (setq-local gh-notify-reason-limit :review-requested)
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-repo (P)
@@ -1300,12 +1199,11 @@ Multiple reason filters can be active at once."
   (gh-notify--filter-notifications))
 
 (defun gh-notify-limit-none ()
-  "Remove all reason filters and show all notifications."
+  "Remove current limit and show all notifications."
   (interactive)
   (cl-assert (eq major-mode 'gh-notify-mode) t)
-  (unless (and (listp gh-notify-reason-limit)
-               (equal gh-notify-reason-limit '(:all)))
-    (setq-local gh-notify-reason-limit '(:all))
+  (unless (eq gh-notify-reason-limit :all)
+    (setq-local gh-notify-reason-limit :all)
     (gh-notify--filter-notifications)))
 
 (defun gh-notify-limit-repo-none ()
@@ -1464,7 +1362,10 @@ If there is a region, only unmark notifications in region."
   (cl-assert (eq major-mode 'gh-notify-mode) t)
   (when-let ((current-notification (gh-notify-current-notification)))
     (let* ((repo (gh-notify-notification-repo current-notification)))
+      ;; XXX: improve me, needs to detect when we don't have a full repo locally
+      ;; XXX: which we can probably just pull from the Forge DB
       (if repo
+          ;; forge-visit no longer exists so use the original implementation here
           (let ((worktree (oref repo worktree)))
             (if (and worktree (file-directory-p worktree))
                 (magit-status-setup-buffer worktree)
@@ -1495,20 +1396,48 @@ Browse issue or PR on prefix P."
         ;; it can result in a lagging point, so take care of all the state rendering
         ;; first, and THEN trigger the buffer switch
 
-        (pcase type
-          ((or 'issue 'pullreq 'discussion)
-           (gh-notify-set-notification-status current-notification 'pending)
-           (with-demoted-errors "Warning visiting topic: %S"
-             (if-let ((topic-obj (gh-notify-notification-topic-obj current-notification)))
-                 (progn
-                   (forge-topic-setup-buffer topic-obj)
-                   ;; Fetch latest topic data
-                   (forge--pull-topic repo topic-obj))
-               (message "Topic %s not in database, try running forge-pull" number))))
-          ('commit
-           (message "Commit notifications not yet supported"))
-          (_
-           (message "Unknown notification type: %s - %s" type title)))))))
+        (cl-letf ((default-directory gh-notify-smokescreen-path))
+
+          ;; XXX: this is a really ugly hack until I figure out how to cleanly make
+          ;; XXX: magit ignore errors when we don't have a local copy of the repo
+          ;; XXX: checked out in our magit paths ... we really don't need a local copy
+          ;; XXX: for interacting with issues and even performing reviews ... e.g.
+          ;; XXX: github-review will work fine with a template magit buffer from a PR
+          ;; XXX: for a non-local repo ...
+
+          ;; XXX: so we throw up a smokescreen with an empty tmp git repo, magit will
+          ;; XXX: fall back to default-directory if it can't find the actual repo ;)
+          ;; XXX: surely there's some non-ganky way to achieve this, but will have to
+          ;; XXX: dig into magit/forge guts a bit more ...
+
+          (unless (file-exists-p default-directory)
+            (make-directory default-directory)
+            (set-file-modes default-directory #o700)
+            (magit-init default-directory))
+
+          (pcase type
+            ('issue
+             (gh-notify-set-notification-status current-notification 'pending)
+             (with-demoted-errors "Warning: %S"
+               (with-temp-buffer
+                 (forge-visit-issue (forge-get-issue topic))
+                 (forge-pull-topic topic))))
+            ('pullreq
+             (gh-notify-set-notification-status current-notification 'pending)
+             (with-demoted-errors "Warning: %S"
+               (with-temp-buffer
+                 (forge-visit-pullreq (forge-get-pullreq topic))
+                 (forge-pull-topic topic))))
+            ('discussion
+             (gh-notify-set-notification-status current-notification 'pending)
+             (with-demoted-errors "Warning: %S"
+               (with-temp-buffer
+                 (forge-visit-discussion (forge-get-discussion topic))
+                 (forge-pull-topic topic))))
+            ('commit
+             (message "Commit not handled yet!"))
+            (_
+             (message "Handling something else (%s) %s\n" type title))))))))
 
 (defun gh-notify-browse-notification (repo-id type number)
   "Browse to a TOPIC of TYPE on GitHub REPO-ID."
